/*	$Id: uname.c,v 1.2 2003/07/30 02:14:22 marius Exp $
/*	$NetBSD: linux_misc.c,v 1.27 1996/05/20 01:59:21 fvdl Exp $	*/

/*
 * Copyright (c) 1995 Frank van der Linden
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed for the NetBSD Project
 *      by Frank van der Linden
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/proc.h>
#include <sys/mount.h>
#include <sys/utsname.h>
#include <sys/queue.h>

#include <sys/syscallargs.h>

#include <compat/linux/linux_types.h>
#include <compat/linux/linux_signal.h>
#include <compat/linux/linux_syscall.h>
#include <compat/linux/linux_syscallargs.h>

#include "uname.h"

int	vmmon_linux_sys_uname __P((struct proc *, void *, register_t *));

char linux_sysname[] = "Linux";
char linux_release[] = "2.4.18";
char linux_version[] = "#0 Sun Apr 1 11:11:11 MET 2000";

extern struct sysent linux_sysent[LINUX_SYS_MAXSYSCALL];
static struct sysent old_ent;

static struct sysent uname_ent = {
	1, sizeof(struct linux_sys_uname_args),
	vmmon_linux_sys_uname
};

extern struct vmware_pid_head *vmware_pidq;

int
vmmon_linux_sys_uname(p, v, retval)
	struct proc *p;
	void *v;
	register_t *retval;
{
	struct linux_sys_uname_args /* {
		syscallarg(struct linux_utsname *) up;
	} */ *uap = v;
	extern char hostname[], machine[], domainname[];
	struct linux_utsname luts;
	struct vmware_pid *pid;
	pid_t ppid;
	struct proc *pp;

	TAILQ_FOREACH(pid, vmware_pidq, next) {
		pp = p;
		do {
			if (pid->pid == pp->p_pid)
				break;
		} while ((pp = pp->p_pptr) != NULL);

		if (pp != NULL)
			break;
	}
	if (pp == NULL) {
		printf("no match for pid %d\n", p->p_pid);
		return (linux_sys_uname(p, v, retval));
	}

	strncpy(luts.l_sysname, linux_sysname, sizeof(luts.l_sysname));
	strncpy(luts.l_nodename, hostname, sizeof(luts.l_nodename));
	strncpy(luts.l_release, linux_release, sizeof(luts.l_release));
	strncpy(luts.l_version, linux_version, sizeof(luts.l_version));
	strncpy(luts.l_machine, machine, sizeof(luts.l_machine));
	strncpy(luts.l_domainname, domainname,
	    sizeof(luts.l_domainname));

	return copyout(&luts, SCARG(uap, up), sizeof(luts));
}

void
vmmon_uname_load(void)
{
	memcpy(&old_ent, &linux_sysent[122], sizeof(old_ent));
	memcpy(&linux_sysent[122], &uname_ent, sizeof(linux_sysent[122]));
}

void
vmmon_uname_unload(void)
{
	memcpy(&linux_sysent[122], &old_ent, sizeof(linux_sysent[122]));
}
